%**************************************************************************
% clear and close
%**************************************************************************
clc;
clear all;
close all;

%**************************************************************************
% parameters governing operation
%**************************************************************************
t_max = 2;                          % total length of time desired (sec)
t_step = 0.001;                     % time increment
time_constant = .1;                 % time constant for envelope detection
threshold = 275;                    % threshold level for thresholding
t_delay = 0.001;                    % time delay during simulation

%**************************************************************************
% produce original signal
%**************************************************************************
t = 0:t_step:t_max;
x = zeros(1,length(t));
for k = 1:length(t)
    if mod(floor(k/(0.1/t_step)),2) == 1
        x(k) = 1;
    else
        x(k) = -1;
    end
end

%**************************************************************************
% generate FM signal
%**************************************************************************
sum = cumsum(x)*t_step;             % integrate the signal
y = cos(2*pi*50*t+50*sum);          % modulate

%**************************************************************************
% display results
%**************************************************************************
figure(1)
set(gcf,'Position',get(0,'Screensize'));
subplot(2,1,1); plot(t,x)
axis([0 t_max -1.25 1.25]);
title('Original Signal');
xlabel('Time (sec)');
ylabel('Signal (volts)');
fprintf('Press any key to continue\n');
pause();
subplot(2,1,2); plot(t,y)
axis([0 t_max -1.25 1.25]);
title('FM Signal');
xlabel('Time (sec)');
ylabel('Signal (volts)');
fprintf('Press any key to continue\n');
pause();

%**************************************************************************
% take derivative using 2 pt. estimate
%**************************************************************************
for k = 1:length(t)-1
    demod(k) = (y(k+1)-y(k))/t_step;
end
demod(length(t)) = demod(length(t)-1);

%**************************************************************************
% recover original signal using peak detection and thresholding
%**************************************************************************
% peak detection
x_demod(1) = 0;
for k = 2:length(t)
    if demod(k) <= 0 && x_demod(k-1) <= 0
        x_demod(k) = 0;
    elseif demod(k) > demod(k-1) && demod(k) > x_demod(k-1)
        x_demod(k) = demod(k);
    else
        x_demod(k) = x_demod(k-1)*exp(-t_step/time_constant);
    end
end
% threshold
x_recov = zeros(size(x_demod));
for k = 1:length(t)
    if x_demod(k) >= threshold
        x_recov(k) = 1;
    else
        x_recov(k) = -1;
    end
end

%**************************************************************************
% display results
%**************************************************************************
figure(2)
set(gcf,'Position',get(0,'Screensize'));
subplot(3,1,1); plot(t,demod)
title('Derivative of FM Signal');
xlabel('Time (sec)');
ylabel('Signal (volts)');
fprintf('Press any key to continue\n');
pause();
subplot(3,1,2); plot(t,x_demod)
title('Envelope-detected Signal');
xlabel('Time (sec)');
ylabel('Signal (volts)');
fprintf('Press any key to continue\n');
pause();
subplot(3,1,3); plot(t,x_recov,'k-',t,x,'r--')
axis([0 t_max -1.25 1.25])
title('Thresholded Signal');
xlabel('Time (sec)');
ylabel('Signal (volts)');
legend('Recovered Signal','Original Signal');
fprintf('Press any key to continue\n');
pause();

%**************************************************************************
% simulation of realtime demodulation
%**************************************************************************
% set up blank data
demod = zeros(size(x));
x_demod = zeros(size(x));
x_recov = zeros(size(x));
y_received = zeros(size(y));
y_received(1) = y(1);
% display figure
figure(3)
set(gcf,'Position',get(0,'Screensize'));
% simulate receiving data
for k = 2:length(t)
    % get data
    y_received(k) = y(k);
    % derivative
    demod(k) = (y_received(k) - y_received(k-1))/t_step;
    % peak detect
    if demod(k) <= 0 && x_demod(k-1) <= 0
        x_demod(k) = 0;
    elseif demod(k) > demod(k-1) && demod(k) > x_demod(k-1)
        x_demod(k) = demod(k);
    else
        x_demod(k) = x_demod(k-1)*exp(-t_step/time_constant);
    end
    % threshold
    if x_demod(k) >= threshold
        x_recov(k) = 1;
    else
        x_recov(k) = -1;
    end
    % update plots
    subplot(4,1,1); plot(t,y_received)
    title('FM Signal');
    xlabel('Time (sec)');
    ylabel('Signal (volts)');
    axis([0 t_max -1.25 1.25]);
    subplot(4,1,2); plot(t,demod)
    title('Derivative of FM Signal');
    xlabel('Time (sec)');
    ylabel('Signal (volts)');
    axis([0 t_max -400 400]);
    subplot(4,1,3); plot(t,x_demod)
    title('Envelope-detected Signal');
    xlabel('Time (sec)');
    ylabel('Signal (volts)');
    axis([0 t_max 0 400]);
    subplot(4,1,4); plot(t,x_recov)
    title('Thresholded Signal');
    xlabel('Time (sec)');
    ylabel('Signal (volts)');
    axis([0 t_max -1.25 1.25]);
    pause(t_delay)
end